#!/bin/bash

# Build, patch, and push all containers

set -ex

FULL=true
CLEAN=false
PROGRESS=plain
if [[ "$1" == "clean" ]]
then
    CLEAN=true
    shift
fi
if [[ "$1" == "tty" ]]
then
    PROGRESS="tty"
    shift
fi
if [[ "$1" == "quick" ]]
then
    FULL=false
    PROGRESS="tty"
    shift
fi

SED_HOST='s/docker build /docker build --add-host "archives.basestation:10.68.1.1" /'

function has_binaries {
    # Check if binaries are available on robotpkg for a target
    if [[ "$1" == "14.04" || "$1" == "16.04" || "$1" == "18.04" || "$1" == "20.04" ]]
    then return 0
    elif [[ "$1" == "dubnium" || "$1" == "erbium" || "$1" == "ferrum" ]]
    then return 0
    elif [[ "$1" == "jessie" || "$1" == "stretch" || "$1" == "buster" ]]
    then return 0
    else return 1
    fi
}

function is_deprecated {
    # Check if binaries are available on robotpkg for a target
    if [[ "$1" == "14.04" || "$1" == "16.04" ]]
    then return 0
    elif [[ "$1" == "dubnium" || "$1" == "erbium" ]]
    then return 0
    else return 1
    fi
}

function is_private {
    # Check if we can publish that
    if [[ "$1" == "dubnium" || "$1" == "erbium" || "$1" == "ferrum" ]]
    then return 0
    else return 1
    fi
}

function build {
    # build all containers for a target
    [[ $(grep -nr "$1" targets | cut -d: -f1) == $(date +%u) || $CLEAN ]] && cache="--no-cache" || cache=""
    $FULL || cache=""
    [[ "$1" == "archlinux" ]] && cache=""
    while read -r registry
    do
        while read -r project
        do
            image="$registry/gepetto/buildfarm/$project"
            if [[ $project == -* || ! -f $image/$1/Dockerfile ]]
            then
                echo "ignoring $image"
            else
                echo -e "\n === build $image === \n"
                docker build --add-host "archives.basestation:10.68.1.1" "--progress=$PROGRESS" $cache -f "$image/$1/Dockerfile" -t "$image:$1" .
            fi
        done < projects
    done < registries
    has_binaries "$1" || return 0
    if $FULL
    then curl "https://rainboard.laas.fr/docker?cmd=build&target__name=$1" | sort -u | sed "$SED_HOST" | bash +e -x || true
    fi
    echo "build done"
}

function dpatch {
    # patch the containers that need it for a target
    has_binaries "$1" || return 0
    is_deprecated "$1" && return 0
    while read -r namespace project robotpkg
    do
        [[ $# == 2 && "$project" != "$2" ]] && continue
        grep -q sot-pattern-generator <<< "$project" && [[ "$1" == "20.04" || "$1" == "stretch" || "$1" == "buster" ]] && continue
        echo -e "\n === patch $namespace -- $project -- $robotpkg === \n"
        docker history --no-trunc "memmos.laas.fr:5000/$namespace/$project/$robotpkg:$1" \
            | head -n2 | grep -q ' /patch' && continue
        docker build --add-host "archives.basestation:10.68.1.1" --progress=$PROGRESS \
            --build-arg REGISTRY="memmos.laas.fr:5000" \
            --build-arg NAMESPACE="$namespace" \
            --build-arg PROJECT="$project" \
            --build-arg ROBOTPKG="$robotpkg" \
            --build-arg DISTRO="$1" \
            -f patches/Dockerfile \
            -t "memmos.laas.fr:5000/$namespace/$project/$robotpkg:$1" .
    done < patches/list
    echo "patch done"
}

function push {
    # push the containers to memmos / gitlab for a target
    [[ "$1" == "archlinux" ]] && return
    while read -r registry
    do
        while read -r project
        do
            image="$registry/gepetto/buildfarm/$project"
            if [[ $project == -* || ! -f $image/$1/Dockerfile ]]
            then
                echo "ignoring $image"
            else
                echo -e "\n === push $image === \n"
                docker push "$image:$1"
            fi
        done < projects
    done < registries
    has_binaries "$1" || return 0
    if $FULL
    then curl "https://rainboard.laas.fr/docker?cmd=push&target__name=$1" | sort -u | bash +e -x || true
    fi
    echo "push done"
}

function hub {
    # tag and push containers to docker hub, for a target
    [[ "$1" == "archlinux" ]] && return
    while read -r registry
    do
        [[ $registry == gitlab* ]] && continue
        while read -r project
        do
            image="$registry/gepetto/buildfarm/$project"
            if [[ $project == -* || ! -f $image/$1/Dockerfile ]]
            then
                echo ignoring "$image"
            else
                echo -e "\n === hub $image === \n"
                docker tag "$image:$1" "gepetto/buildfarm:$project-$1"
                docker push "gepetto/buildfarm:$project-$1"
            fi
        done < projects
    done < registries
    echo "done"
}


if [[ $# == 1 ]]
then
    build "$1"
    $FULL && dpatch "$1"
    push "$1"
    is_private "$1" || hub "$1"
    echo "buildfarm complete for $1"
else
    rm -rf logs
    mkdir logs
    while read -r target
    do
        (
        build "$target" > "logs/build_$target" 2>&1
        $FULL && dpatch "$target" > "logs/patch_$target" 2>&1
        push "$target" > "logs/push_$target" 2>&1
        is_private "$target" || hub "$target" > "logs/hub_$target" 2>&1
        ) &
    done < targets
    wait
    echo "buildfarm complete"
fi
